<?php

namespace App\Http\Controllers;

use the;
use Exception;
use App\Models\Order;
use GuzzleHttp\Client;
use App\Models\Session;
use setasign\Fpdi\Fpdi;
use App\Models\AccessDetail;
use App\Models\ReturnDetail;
use Illuminate\Http\Request;
use App\Models\ReturnOrderDetail;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use libphonenumber\PhoneNumberType;
use libphonenumber\PhoneNumberUtil;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Storage;
use libphonenumber\NumberParseException;
use GuzzleHttp\Exception\ClientException;


class ReturnController extends Controller
{
    private function createClient($shop, $accessToken)
    {

        $baseUri = "https://{$shop}/admin/api/2023-04/";

        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
    public function submitReturnDetails(Request $request, $domain)
    {
        $domainName = $domain;
        $orderIdReceived = $request->query('order_id');
        $emailId = $request->get('email_id');
        $orderId = ltrim($orderIdReceived, '#');
    
        $shop_details = AccessDetail::where('domain_name', $domainName)->first();
        $shop_id = $shop_details->shop_id;
        $accessTokenDetails = Session::where('id', $shop_id)->first();
        $accessToken = $accessTokenDetails->access_token;
        $shop = $accessTokenDetails->shop;
        $client = $this->createClient($shop, $accessToken);
    
        $response = $client->get("orders.json?status=any&name=$orderId");
        $result = json_decode($response->getBody(), true);
    
        if (empty($result['orders'])) {
            return response()->json([
                'success' => false,
                'message' => 'Order not found.',
            ]);
        }
    
        $order = $result['orders'][0]; 
    
        $customerEmail = $order['email'] ?? '';
        $customerPhone = $order['phone'] ?? '';
    
        if ($customerEmail !== $emailId && $customerPhone !== $emailId) {
            return response()->json([
                'success' => false,
                'message' => 'Email or phone number does not match the order.',
            ]);
        }
    
        $orderMainId = $order['id'];
        $response = $client->get("orders/{$orderMainId}/fulfillment_orders.json");
        $fulfillmentOrders = json_decode($response->getBody(), true);
    
        $fulfilledItems = [];
    
        foreach ($fulfillmentOrders['fulfillment_orders'] as $fulfillmentOrder) {
            foreach ($fulfillmentOrder['line_items'] as $lineItem) {
                $lineItemQuantity = $lineItem['quantity'] ?? 0;
                $lineItemFulfillableQuantity = $lineItem['fulfillable_quantity'] ?? 0;
                $fulfilledQuantity = $lineItemQuantity - $lineItemFulfillableQuantity;
    
                if ($fulfilledQuantity > 0) {
                    $lineItemId = $lineItem['line_item_id'];
    
                    foreach ($order['line_items'] as $originalLineItem) {
                        if ($originalLineItem['id'] == $lineItemId) {
                            $fulfilledItems[] = array_merge($originalLineItem, ['fulfilled_quantity' => $fulfilledQuantity]);
                            break;
                        }
                    }
                }
            }
        }
    
        if (!empty($fulfilledItems)) {
            // Check for returned items and quantity differences
            $nonReturnedItems = [];
            $returnOrders = ReturnOrderDetail::where('order_id', $orderMainId)->get(); // Retrieve all return orders for the same order ID
        
            if (!$returnOrders->isEmpty()) {
                // Create an associative array to store the total returned quantity for each lineItemId
                $returnedQuantities = [];
        
                foreach ($returnOrders as $returnOrder) {
                    $returnLineItems = json_decode($returnOrder->returnLineItem_details, true); // Decode JSON
        
                    foreach ($returnLineItems as $returnLineItemGroup) {
                        foreach ($returnLineItemGroup as $returnLineItem) {
                            $lineItemId = $returnLineItem['lineItemId'];
                            $quantity = $returnLineItem['quantity'];
        
                            if (isset($returnedQuantities[$lineItemId])) {
                                $returnedQuantities[$lineItemId] += $quantity; // Sum the quantities for the same lineItemId
                            } else {
                                $returnedQuantities[$lineItemId] = $quantity;
                            }
                        }
                    }
                }
        
                foreach ($fulfilledItems as $fulfilledItem) {
                    $fulfilledItemId = $fulfilledItem['id'];
                    $fulfilledQuantity = $fulfilledItem['fulfilled_quantity'];
        
                    // Get the total returned quantity for this lineItemId
                    $returnQuantity = $returnedQuantities[$fulfilledItemId] ?? 0;
        
                    // Calculate the quantity difference
                    $quantityDifference = $fulfilledQuantity - $returnQuantity;
        
                    // If the quantity difference is not zero, include this item
                    if ($quantityDifference != 0) {
                        $nonReturnedItems[] = array_merge($fulfilledItem, [
                            'returned_quantity' => $returnQuantity,
                            'quantity_difference' => $quantityDifference
                        ]);
                    }
                }
        
                if (!empty($nonReturnedItems)) {
                    return response()->json([
                        'success' => true,
                        'orderNumber' => $order['order_number'],
                        'orderData' => $nonReturnedItems,
                    ]);
                } else {
                    return response()->json([
                        'success' => false,
                        'message' => 'No items found to return.',
                    ]);
                }
            } else {
                // No previous returns found, so all fulfilled items are non-returned
                foreach ($fulfilledItems as &$fulfilledItem) {
                    $fulfilledItem['quantity_difference'] = $fulfilledItem['fulfilled_quantity'];
                    $fulfilledItem['returned_quantity'] = 0; // No items have been returned yet
                }
        
                return response()->json([
                    'success' => true,
                    'orderNumber' => $order['order_number'],
                    'orderData' => $fulfilledItems,
                ]);
            }
        } else {
            return response()->json([
                'success' => false,
                'message' => 'No fulfilled items found for the order.',
            ]);
        }
        
    }
    
    public function submitReturn(Request $request, $domain)
    {
        $domainName = $domain;
        $orderId = $request->input('orderId');
        $items = json_decode($request->input('items'), true);

        $shop_details = AccessDetail::where('domain_name', $domainName)->first();
        $shop_id = $shop_details->shop_id;
        $access_id = $shop_details->id;
        $accessTokenDetails = Session::where('id', $shop_id)->first();
        $accessToken = $accessTokenDetails->access_token;
        $shop = $accessTokenDetails->shop;
        $client = $this->createClient($shop, $accessToken);

        $response = $client->get("orders.json?status=any&name=$orderId");
        $result = json_decode($response->getBody(), true);

        if (empty($result['orders'])) {
            return response()->json([
                'success' => false,
                'message' => 'Order not found.',
            ]);
        }

        $order = $result['orders'][0]; 
        $orderMainId = $order['id'];
        $email = $order['email'];
        $phone = $order['phone'];
        $customer_first_name = $order['customer']['first_name'];
        $customer_last_name = $order['customer']['last_name'];
       
        $customer_name = $customer_first_name . ' ' . $customer_last_name;
        $variantData = [];
        
        foreach ($order['line_items'] as $item) {

            $productId = $item['product_id'];
            $response = $client->get("products/$productId.json");
            $productResult = json_decode($response->getBody(), true);
            
            foreach ($productResult['product']['variants'] as $variant) {
                if($item['variant_id']==$variant['id'])
                {
                $product_id = $productResult['product']['id'];
                $inventoryItemId = $variant['inventory_item_id'];
                $inventoryResponse = $client->get("inventory_items/$inventoryItemId.json");
                $inventoryItem = json_decode($inventoryResponse->getBody(), true);
               
                $variantData[] = [
                    'product_id' => $product_id,
                    'harmonized_system_code' => $inventoryItem['inventory_item']['harmonized_system_code'] ?? null,
                ];
                 //dd($variantData);
            }
            }
        }

        $response = $client->post("https://{$shop}/admin/api/2023-01/graphql.json", [
            'headers' => [
                'X-Shopify-Access-Token' => $accessToken,
                'Content-Type' => 'application/graphql',
            ],
            'body' => <<<GRAPHQL
            query returnableFulfillmentsQuery {
             returnableFulfillments(orderId: "gid://shopify/Order/{$orderMainId}", first: 10) {
                edges {
                    node {
                        id
                        fulfillment {
                            id
                        }
                        returnableFulfillmentLineItems(first: 10) {
                            edges {
                                node {
                                    fulfillmentLineItem {
                                        id
                                        lineItem {
                                            id
                                        }
                                    }
                                    quantity
                                }
                            }
                        }
                    }
                }
            }
        }
        GRAPHQL
        ]);

        $body = $response->getBody();
        $data = json_decode($body->getContents(), true);

        if (!$orderId || !$items || empty($items)) {
            return response()->json([
                'success' => false,
                'message' => 'Invalid input data',
            ], 400);
        }

        $returnResponses = [];
        $fulfillmentLineItems = [];

        foreach ($items as $item) {

            $itemId = $item['id'];
            $itemQuantity = $item['quantity'];
            $itemReason = $item['reason'];
            $returnReasonNote = $item['returnReasonNote'];

            foreach ($data['data']['returnableFulfillments']['edges'] as $edge) {
                foreach ($edge['node']['returnableFulfillmentLineItems']['edges'] as $lineItemEdge) {
                    $lineItem = $lineItemEdge['node'];
                    if ($lineItem['fulfillmentLineItem']['lineItem']['id'] === "gid://shopify/LineItem/{$itemId}") {
                        $fulfillmentLineItems[] = [
                            'fulfillmentLineItemId' => $lineItem['fulfillmentLineItem']['id'],
                            'quantity' => min($itemQuantity, $lineItem['quantity']),
                            'returnReason' => $this->mapReturnReason($itemReason),
                            'returnReasonNote' => $returnReasonNote,
                        ];

                        $itemQuantity -= $lineItem['quantity'];
                        if ($itemQuantity <= 0) {
                            break 2;
                        }
                    }
                }
            }
        }
       
            try {
                $allReturnRequests = [];
                $allProductReturnLineItems = [];

                foreach ($items as $item) {
                    $itemId = $item['id'];
                    $itemQuantity = $item['quantity'];
                    $itemReason = $item['reason'];
                    $customerNote = $item['returnReasonNote'];
                    $totalReturnAmount = $item['totalReturnAmount'];
                    $productPrice = $item['productPrice'];
                    $productName = $item['productName'];

                   
                    $productReturnLineItems = [];
                    $ReturnLineItems = [];
                    

                    foreach ($data['data']['returnableFulfillments']['edges'] as $edge) {
                        foreach ($edge['node']['returnableFulfillmentLineItems']['edges'] as $lineItemEdge) {
                            $lineItem = $lineItemEdge['node'];
                            if ($lineItem['fulfillmentLineItem']['lineItem']['id'] === "gid://shopify/LineItem/{$itemId}") {
                                $productReturnLineItems[] = [
                                    'fulfillmentLineItemId' => $lineItem['fulfillmentLineItem']['id'],
                                    'quantity' => min($itemQuantity, $lineItem['quantity']),
                                    'returnReason' => $this->mapReturnReason($itemReason),
                                    'customerNote' => $customerNote                                  
                                ];

                                $ReturnLineItems[] = [
                                    'fulfillmentLineItemId' => $lineItem['fulfillmentLineItem']['id'],
                                    'quantity' => min($itemQuantity, $lineItem['quantity']),
                                    'returnReason' => $this->mapReturnReason($itemReason),
                                    'customerNote' => $customerNote,
                                    'productPrice' => $productPrice,
                                    'productName' => $productName,
                                    'lineItemId' => $itemId
                                ];

                                $itemQuantity -= $lineItem['quantity'];
                                if ($itemQuantity <= 0) {
                                    break 2;
                                }
                            }
                        }
                       
                    }
                    
                    $allProductReturnLineItems[] = $ReturnLineItems;
 
                    $response = $client->post("https://{$shop}/admin/api/2023-01/graphql.json", [
                        'headers' => [
                            'X-Shopify-Access-Token' => $accessToken,
                            'Content-Type' => 'application/json',
                        ],
                        'json' => [
                            'query' => <<<GRAPHQL
                            mutation ReturnRequest(\$input: ReturnRequestInput!) {
                                returnRequest(input: \$input) {
                                    userErrors {
                                        field
                                        message
                                    }
                                    return {
                                        id
                                        status
                                        returnLineItems(first: 10) {
                                            edges {
                                                node {
                                                    id
                                                    returnReason
                                                    customerNote
                                                }
                                            }
                                        }
                                        order {
                                            id
                                        }
                                    }
                                }
                            }
                            GRAPHQL,
                            'variables' => [
                                'input' => [
                                    'orderId' => "gid://shopify/Order/{$orderMainId}",
                                    'returnLineItems' => $productReturnLineItems
                                ]
                            ]
                        ]
                    ]);

                    $responseData = json_decode($response->getBody()->getContents(), true);
                    if (isset($responseData['data']['returnRequest']['return']['status']) && $responseData['data']['returnRequest']['return']['status'] === 'REQUESTED') {
                        $allReturnRequests[] = $responseData['data']['returnRequest']['return'];
                    } else {
                      
                        $allReturnRequests[] = [
                            'error' => true,
                            'productId' => $itemId,
                            'message' => 'Return request failed for this product',
                            'errors' => $responseData['data']['returnRequest']['userErrors'] ?? [],
                        ];
                    }
                }

                // Save all return details to database as one entry
                if (!empty($allReturnRequests)) {
                    $allRequestedReturns = array_filter($allReturnRequests, function($request) {
                        return isset($request['status']) && $request['status'] === 'REQUESTED';
                    });
                    
                    if (!empty($allRequestedReturns)) {
                        $returnDetail = new ReturnOrderDetail();
                        $returnDetail->access_id = $access_id;
                        $returnDetail->order_number = $orderId;
                        $returnDetail->order_id = $orderMainId;
                        $returnDetail->email = $email;
                        $returnDetail->phone = $phone;
                        $returnDetail->customer_first_name =$customer_name;
                        $returnDetail->totalReturnAmount = $totalReturnAmount;
                        $returnDetail->order_data = json_encode($order);
                        $returnDetail->hs_code_details = json_encode($variantData);
                        $returnDetail->return_details = json_encode($allRequestedReturns);
                        $returnDetail->returnLineItem_details = json_encode($allProductReturnLineItems);
                        $returnDetail->save();
                
                        return response()->json([
                            'success' => true,
                            'message' => 'Return requests processed successfully',
                            'data' => $allRequestedReturns,
                        ]);
                    } else {
                        return response()->json([
                            'success' => false,
                            'message' => 'No return requests were successful',
                        ]);
                    }
                } else {
                    return response()->json([
                        'success' => false,
                        'message' => 'No return requests were processed',
                    ]);
                }

            } catch (\Exception $e) {
                return response()->json([
                    'success' => false,
                    'message' => 'Error submitting return requests: ' . $e->getMessage(),
                ]);
            }
    }

    private function mapReturnReason($reason)
    {
        $map = [
            'UNKNOWN' => 'UNKNOWN',
            'SIZE_TOO_SMALL' => 'SIZE_TOO_SMALL',
            'SIZE_TOO_LARGE' => 'SIZE_TOO_LARGE',
            'UNWANTED' => 'UNWANTED',
            'NOT_AS_DESCRIBED' => 'NOT_AS_DESCRIBED',
            'WRONG_ITEM' => 'WRONG_ITEM',
            'DEFECTIVE' => 'DEFECTIVE',
            'STYLE' => 'STYLE',
            'COLOR' => 'COLOR',
            'OTHER' => 'OTHER',

        ];
        return $map[$reason] ?? 'UNKNOWN'; 
    }


    public function returnApproveSubmit(Request $request)
{
    $lineItemId = $request->input('lineItemId');
    $returnId = $request->input('returnId');
    $orderNumber =  $request->input('orderNumber');
    $orderId =  $request->input('orderId');

    // Retrieve the return order details based on the order number
    $returnTable_details = ReturnOrderDetail::find($orderId);

    if (!$returnTable_details) {
        return response()->json([
            'success' => false,
            'message' => 'Return order not found'
        ], 404);
    }

    $access_id = $returnTable_details->access_id;
    $shop_details = AccessDetail::where('id', $access_id)->first();
    $shop_id = $shop_details->shop_id;
    $accessTokenDetails = Session::where('id', $shop_id)->first();
    $accessToken = $accessTokenDetails->access_token;
    $shop = $accessTokenDetails->shop;
    $client = $this->createClient($shop, $accessToken);

    // Approve the return request via the Shopify API
    $response = $client->post("https://{$shop}/admin/api/2023-01/graphql.json", [
        'headers' => [
            'X-Shopify-Access-Token' => $accessToken,
            'Content-Type' => 'application/json',
        ],
        'json' => [
            'query' => <<<GRAPHQL
            mutation ApproveReturnRequestMutation(\$input: ReturnApproveRequestInput!) {
                returnApproveRequest(input: \$input) {
                    return {
                        id
                        status
                    }
                    userErrors {
                        field
                        message
                    }
                }
            }
            GRAPHQL,
            'variables' => [
                'input' => [
                    'id' => "gid://shopify/Return/{$returnId}"
                ]
            ]
        ]
    ]);

    $body = $response->getBody();
    $result = json_decode($body->getContents(), true);

    if (isset($result['data']['returnApproveRequest']['return'])) {
        $approvedReturn = $result['data']['returnApproveRequest']['return'];
        $approvedReturn['lineItemId'] = $lineItemId;
        
        $returnTable_details = ReturnOrderDetail::find($orderId);
               
                if (!empty($returnTable_details->approve_decline_details)) {
                    $existingData = json_decode($returnTable_details->approve_decline_details, true);
                    
                    if (!is_array($existingData)) {
                        $existingData = [$existingData];
                    }
                   
                    $existingData[] = $approvedReturn;
                    
                    $newApproveDeclineDetails = json_encode($existingData);
                } else {
                    // If there's no existing data, just encode the new data
                    $newApproveDeclineDetails = json_encode([$approvedReturn]);
                }

                $returnTable_details->approve_decline_details = $newApproveDeclineDetails;
                $returnTable_details->update();

                return response()->json([
                    'success' => true,
                    'message' => 'Return approved successfully',
                    'data' => $approvedReturn
                ]);
            } else {
                $errors = $result['data']['returnApproveRequest']['userErrors'] ?? 'Unknown error occurred';
                return response()->json([
                    'success' => false,
                    'message' => 'Failed to approve return',
                    'errors' => $errors
                ], 400);
            }
}


    public function returnDeclineSubmit(Request $request)
    {
        $lineItemId = $request->input('lineItemId');
        $declineReason = $request->input('declineReason');
        $returnId = $request->input('returnId');
        $orderNumber =  $request->input('orderNumber');
        $oderId=$request->input('orderId');
     
        $returnTable_details = ReturnOrderDetail::find($oderId);
        $access_id = $returnTable_details->access_id;
        $shop_details = AccessDetail::where('id', $access_id)->first();
        $shop_id = $shop_details->shop_id;
        $accessTokenDetails = Session::where('id', $shop_id)->first();
        $accessToken = $accessTokenDetails->access_token;
        $shop =  $accessTokenDetails->shop;
        $client = $this->createClient($shop, $accessToken);

        $response = $client->post("https://{$shop}/admin/api/2023-01/graphql.json", [
            'headers' => [
                'X-Shopify-Access-Token' => $accessToken,
                'Content-Type' => 'application/json',
            ],
            'json' => [
                'query' => <<<GRAPHQL
                mutation DeclineReturnRequestMutation(\$input: ReturnDeclineRequestInput!) {
                    returnDeclineRequest(input: \$input) {
                        return {
                            id
                            status
                            decline {
                                reason
                            }
                        }
                        userErrors {
                            field
                            message
                        }
                    }
                }
                GRAPHQL,
                'variables' => [
                    'input' => [
                        'id' => "gid://shopify/Return/{$returnId}",
                        'declineReason' => $declineReason
                    ]
                ]
            ]
        ]);
        
        $body = $response->getBody();
        $result = json_decode($body->getContents(), true);
        
        if (isset($result['data']['returnDeclineRequest']['return'])) {
            $declinedReturn = $result['data']['returnDeclineRequest']['return'];
            $declinedReturn['lineItemId'] = $lineItemId;

            $returnTable_details = ReturnOrderDetail::find($oderId);
               
                if (!empty($returnTable_details->approve_decline_details)) {
                    $existingData = json_decode($returnTable_details->approve_decline_details, true);
                   
                    if (!is_array($existingData)) {
                        $existingData = [$existingData];
                    }
                   
                    $existingData[] = $declinedReturn;
                    
                    $newApproveDeclineDetails = json_encode($existingData);
                } else {
                    // If there's no existing data, just encode the new data
                    $newApproveDeclineDetails = json_encode([$declinedReturn]);
                }

                $returnTable_details->approve_decline_details = $newApproveDeclineDetails;
                $returnTable_details->update();

            return response()->json([
                'success' => true,
                'message' => 'Return declined',
                'data' => $declinedReturn
            ]);
        } else {
            $errors = $result['data']['returnDeclineRequest']['userErrors'] ?? 'Unknown error occurred';
            return response()->json([
                'success' => false,
                'message' => 'Failed to decline return',
                'errors' => $errors
            ], 400);
        }

    }

    public function getAWBNumberReturn(Request $request)
    {
        $orderId = $request->input('orderId');
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
                       ->where('access_token', $accessToken)
                       ->first();
        if (!$shop) {
            return response()->json(['error' => 'Session not found'], 404);
        }
        
        $accessDetail = $shop->accessDetail;
    
        $order = ReturnOrderDetail::where('access_id', $accessDetail->id)
                      ->where('id', $orderId)
                      ->first();
        //dd($order);
        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }
    
        try {
            $awb_number = $this->getShipmentDetails($accessDetail->id, $order, $orderId);
    
            $pushed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 1)
                                  ->orderBy('created_at', 'desc')
                                  ->get();
            
            $orders = Order::where('access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $removed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 0)
                                  ->where('remove_status', 1)
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            return response()->json(['orders' => $orders, 'pushed_orders' => $pushed_orders, 'removed_orders' => $removed_orders], 200);
        } catch (\Exception $e) {
           
            Log::error('Error in getAWBNumber: ' . $e->getMessage());
    
            $orders = Order::where('access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $pushed_orders = Order::where('access_id', $accessDetail->id)
                                  ->where('status', 1)
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            $removed_orders = Order::where('access_id', $accessDetail->id)
                                   ->where('status', 0)
                                   ->where('remove_status', 1)
                                   ->orderBy('created_at', 'desc')
                                   ->get();
    
            return response()->json([
                'error' => $e->getMessage(),
                'orders' => $orders,
                'pushed_orders' => $pushed_orders,
                'removed_orders' => $removed_orders
            ], 500);
        }
    }
    private function getShipmentDetails($id,$order,$orderId)
    {  
                $accessDetail = AccessDetail::find($id);
                $session = $accessDetail->sessions;
            
                    if (!$accessDetail) {
                        throw new \Exception('AccessDetail not found');
                    }

                    $token = $accessDetail->api_key;
                    $customer_code = $accessDetail->customer_code;

                    $defaultWeight=$accessDetail->weight;

                    $order_data=$order->order_data;
                    $order_data = json_decode($order_data); 

                    if (!$order || !$order_data) {
                        throw new \Exception('Order data not found or empty');
                    }
                    
                    $shipping_address = $order_data->shipping_address;
                
                    if (!$shipping_address) {
                        throw new \Exception('Shipping address not found or empty');
                    }


            $order_items = [];
            $order_packages = [];
            $line_items = $order_data->line_items;
            $order_id = $order->order_id;
            $hs_codes=$order->hs_code_details;
            $hs_codes = json_decode($hs_codes);
            $total_weight = 0;
            $contents = [];
            
            $approve_decline_details = json_decode($order->approve_decline_details, true);

            if (!is_array($approve_decline_details)) {
                throw new \Exception('Approve decline details are not valid');
            }
            $awb_number_details = json_decode($order->awb_number_details, true) ?? [];

            if (!is_array($awb_number_details)) {
                throw new \Exception('Awb number details are not valid');
            }
        
                //dd($hs_codes);
             foreach ($line_items as $line_item) {
                $process_item = false;
                $existing_awb = false;

                // Check approve_decline_details
                foreach ($approve_decline_details as $approve_decline_detail) {
                    if ($approve_decline_detail['lineItemId'] == $line_item->id && $approve_decline_detail['status'] == 'OPEN') {
                        $process_item = true;
                        break;
                    }
                }

                if (!empty($awb_number_details)) {
                    foreach ($awb_number_details as $awb_detail) {
                        if ($awb_detail['line_item_id'] == $line_item->id) {
                            $existing_awb = true;
                            break;
                        }
                    }
                }

                // Process the item if it's open and doesn't have an existing AWB
                if ($process_item && !$existing_awb) {
                    if (empty($line_item->price) || empty($line_item->name)) {
                        throw new \Exception('Product data incomplete');
                    }
                    $assigned_hscode = "00000000";
                    
                    foreach ($hs_codes as $hs_code) {
                        if ($hs_code->product_id == $line_item->product_id) {
                            $assigned_hscode = $hs_code->harmonized_system_code ?? "00000000";
                            break;
                        }
                    }

                    if (empty($assigned_hscode)) {
                        throw new \Exception('HS code is wrong');
                    }


                    $weight = round((float)$line_item->grams / 1000, 2);
                    $total_weight += $weight;

                    $order_items[] = [
                        //"weight" => round((float)$line_item->grams / 1000, 2),
                        "weight" => $weight,
                        "qty" => (int)$line_item->fulfillable_quantity,
                        "price" => (float)$line_item->price,
                        "is_dangerous" => false,
                        "assigned_hscode" => (string)$assigned_hscode,
                        "content" => $line_item->name
                    ];

                    $contents[] = $line_item->name;

                }
                
            }

                    if ($total_weight == 0) {
                        $total_weight = $defaultWeight;
                    }
                    
                    $concatenated_content = implode(", ", $contents);

                    $order_packages[] = [
                        "barcode_value" => "",
                        "content" => $concatenated_content,
                        //"content" => $line_item->name,
                        "height" => 1,
                        "length" => 1,
                        "weight" => round((float)($total_weight), 2),
                        //"weight" => round((float)($line_item->grams != 0 ? $line_item->grams / 1000 : $defaultWeight), 2),
                        "width" => 1
                    ];
               

            $closing_time = $accessDetail->closing_time;
            $ready_at_time = $accessDetail->ready_at_time;
            $payment_gateway=$order_data->payment_gateway_names;

            $payment_terms=$order_data->payment_terms;

            $cod_value = 0.0; 

            if (empty($payment_gateway) && $payment_terms) {
                $cod_value = (float)$order_data->current_total_price;
            
            }

            if (is_array($payment_gateway) && in_array("Cash on Delivery (COD)", $payment_gateway)) {
                $cod_value = (float)$order_data->current_total_price;

            }


            if (empty($shipping_address->address1) ||  empty($shipping_address->country_code)) {
                throw new \Exception('Consignee information incomplete');
            }
            if(empty($shipping_address->phone)){
                throw new \Exception('Consignee phone number is empty');
            }
            if(empty($shipping_address->first_name)){
                throw new \Exception('Consignee name is empty');
            }
            if(empty($shipping_address->city)){
                throw new \Exception('Consignee city is empty');
            }
            $countryCode = $shipping_address->country_code;

            if($countryCode=="AE")
            {
            $service_type = $accessDetail->product_group1;
            $product = $accessDetail->product_type1;
            $service = $accessDetail->services1;
            }
            else{
                $service_type ="Export";
                $product = $accessDetail->product_type2;
                $service = $accessDetail->services2;
            }

            $phoneUtil = PhoneNumberUtil::getInstance();
                
                try {
                    $exampleNumber = $phoneUtil->getExampleNumberForType($countryCode, PhoneNumberType::MOBILE);
                
                    $countryCallingCode =(string)$exampleNumber->getCountryCode();
                
                    
                } catch (NumberParseException $e) {
                    throw new \Exception('Phone number invalid');
                }

                $address= $shipping_address->address1;
    
   if (!empty($shipping_address->address2)) {
        $address .= ', Apartment: ' . $shipping_address->address2;
    }
    if (!empty($shipping_address->city) && $shipping_address->city != '-') {
        $address .= ', ' . $shipping_address->city;
    }
    
    $city=$shipping_address->city;

    if (!empty($shipping_address->city) && $shipping_address->city != '-') {
        
        $city = $shipping_address->city;
     
    } 
    else{
        $city =$shipping_address->province;
    }
   
  
    $landmark = $shipping_address->address1;
   
    if (!empty($city) && $city != '-') {
        $landmark .= ', ' . $city;
    } elseif (!empty($shipping_address->province) && $shipping_address->province != '-') {
        $landmark .= ', ' . $shipping_address->province;
    }

    if($shipping_address->company)
    {
        $company_name=$shipping_address->company;
    }
    else{
        $company_name="N/A";
    }

    if ($shipping_address->first_name || $shipping_address->last_name) {
        if ($shipping_address->first_name && $shipping_address->last_name) {
            $primary_contact_person = $shipping_address->first_name . ' ' . $shipping_address->last_name;
        } elseif ($shipping_address->first_name) {
            $primary_contact_person = $shipping_address->first_name;
        } else {
            $primary_contact_person = $shipping_address->last_name;
        }
    }
  
    

    $consignee_information = [
    "address" =>  $address,
    "city" => $city,
    "company_name" => $company_name,
    "landmark" => $landmark,
    "phone_code" =>$countryCallingCode,
    "primary_contact_number" => $shipping_address->phone,
    "primary_contact_person" => $primary_contact_person,
    "secondary_contact_number" => "",
    "secondary_contact_person" => "",
    "postcode" => $shipping_address->zip ?? "",
    "country" => $shipping_address->country_code
];

            //$consignment_value = 49;
            $insurance_value = 1;
            $is_reverse_shipment = true;

            $shipper_landMark= $accessDetail->landmark;
            
            if (!empty($accessDetail->city) && $accessDetail->city != '-') {
                $shipper_landMark .= ', ' . $accessDetail->city;
            } 

            $shipper_information = [
                "address" => $accessDetail->adress_line,
                "city" => $accessDetail->city,
                "company_name" => $accessDetail->company_name,
                "landmark" => $shipper_landMark,
                "phone_code" => $accessDetail->primary_contact_number_code,
                "primary_contact_number" => $accessDetail->primary_contact_number,
                "primary_contact_person" => $accessDetail->primary_contact_person,
                "secondary_contact_number" => $accessDetail->secondary_contact_number??"",
                "secondary_contact_person" => $accessDetail->secondary_contact_person??"",
                "country" => $accessDetail->country,
                "postcode" => $accessDetail->zipcode??""
            ];
            
            try {
                $response = Http::withHeaders([
                    'token' => $token,
                    'Accept' => 'application/json',
                ])->post('https://ferointegration.feroai.com/api/v1/integration/shipment/', [
                    "closing_time" => $closing_time,
                    "cod_value" => $cod_value,
                    "service_type" =>$service_type,
                    "awb_number" => "",
                    "consignee_information" => $consignee_information,
                    "customer" =>  $customer_code,
                    "insurance_value" => $insurance_value,
                    "is_reverse_shipment" => $is_reverse_shipment,
                    "order_items" =>  $order_items,
                    "order_packages" => $order_packages,
                    "order_reference_number" => "",
                    "product" => $product,
                    "ready_at_time" => $ready_at_time,
                    "service" => $service,
                    "shipper_information" => $shipper_information,
                    "shipper_reference_number" => "",
                    "special_instruction" => "A"
                ]);

                $response_body = $response->json();
                Log::info($response_body);

                if ($response->successful()) {
                    $awb_number = $response_body['detail']['awb_number'] ?? null;
                    $awb_status=$response_body['detail']['status'] ?? null;
                    Log::info($awb_number);
                    if ($awb_number && $awb_status !='Pending') {

                        $combined_data_array = [];

                        foreach ($line_items as $line_item) {
                            $existing_awb = false;
                    
                            // Check if this line item already has an AWB number
                            foreach ($awb_number_details as $awb_detail) {
                                if ($awb_detail['line_item_id'] == $line_item->id) {
                                    $existing_awb = true;
                                    break;
                                }
                            }
                    
                            // Only process if there's no existing AWB for this line item
                            if (!$existing_awb) {
                                foreach ($approve_decline_details as $approve_decline_detail) {
                                    if ($approve_decline_detail['lineItemId'] == $line_item->id && $approve_decline_detail['status'] == 'OPEN') {
                                        // Add data for each matching line item to the array
                                        $combined_data_array[] = [
                                            'awb_number' => $awb_number,
                                            'line_item_id' => $line_item->id
                                        ];
                                    }
                                }
                            }
                        }
                    
                        // Only proceed if there are new items to add
                        if (!empty($combined_data_array)) {
                            // Find the existing ReturnOrderDetail
                            $returnDetail = ReturnOrderDetail::where('access_id', $accessDetail->id)
                                                             ->where('id', $orderId)
                                                             ->first();
                    
                            if ($returnDetail) {
                              
                                $existing_details = json_decode($returnDetail->awb_number_details, true) ?: [];                               
                                $merged_details = array_merge($existing_details, $combined_data_array);                               
                                $returnDetail->awb_number_details = json_encode($merged_details);
                                $returnDetail->save();
                            } else {
                                // Create a new record if it doesn't exist
                                $returnDetail = new ReturnOrderDetail();
                                $returnDetail->access_id = $accessDetail->id;
                                $returnDetail->order_id = $order_id;
                                $returnDetail->awb_number_details = json_encode($combined_data_array);
                                $returnDetail->save();
                            }
                    

                        }
                        // $track_response = Http::withHeaders([
                        //     'token' => $token,
                        //     'Accept' => 'application/json',
                        // ])->post('https://ferointegrationuat.feroai.com/api/v1/integration/awb_track/', [
                        //     "awb_numbers" => $awb_number]);
                        
                        // Log::info($track_response);
                        // $jsonResponse = $track_response->json();
                        // if (!$jsonResponse || !isset($jsonResponse['success']) || !$jsonResponse['success']) {
                        
                        //     return response()->json(['error' => 'AWB status found in response'], 500);
                        // }
                        // //dd($jsonResponse);
                        // $detail = $jsonResponse['detail'][0];
                        // $awbLogs = $detail['awb_logs'];
                        // $awb_status = $awbLogs[0]['status'];
                        // Log::info($awb_status);
                        // $order->awb_no = $awb_number;
                        // $order->pickup_status=$awb_status;
                        // $order->status = 1;
                        // $order->save();
                   
                    

                        return response()->json(['awb_number' => $awb_number], 200);
                    } else {
                        $errorMessage=$response_body['detail']['warning']??'Order is pending';
                        throw new \Exception($errorMessage);
                       // return response()->json(['error' => 'AWB number not found in response'], 500);
                    }
                } else {
                    $errorMessage = $response_body['errors'][1]['error_message'] ?? 'Unknown error occurred.';
                    throw new \Exception($errorMessage);
                    //return response()->json(['error' => $errorMessage], $response->status());
                }
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
                //return response()->json(['error' => $e->getMessage()], 500);
            }
    }


   
public function getDownloadLabelReturn(Request $request)
{
        $lineItemId=$request->input('lineItemId');
       
        $id=$request->input('orderId');
         //dd($id);
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
            ->where('access_token', $accessToken)
            ->first();

            if (!$shop) {
                return response()->json(['error' => 'Session not found'], 404);
            }

        
        $accessDetail = $shop->accessDetail;
        $pdf_size=(string)$accessDetail->pdf_size;

        $token = $accessDetail->api_key;

        $download_order = ReturnOrderDetail::where('access_id', $accessDetail->id)
                                            ->where('id', $id)
                                            ->first();

        if($download_order)
        {
            $awbNoDetails = json_decode($download_order->awb_number_details, true);
            $awbNo = null;
            foreach ($awbNoDetails as $detail) {
                if ($detail['line_item_id'] == $lineItemId) {
                    $awbNo = $detail['awb_number'];
                    break;
                }
            }

            $download_response = Http::withHeaders([
                'token' => $token,
                'Accept' => 'application/json',
            ])->post('https://ferointegration.feroai.com/api/v1/integration/awb_pdf/', [
                "awb_number" =>  $awbNo,
                "pdf_size" => $pdf_size
            
            ]);

                Log::info($download_response);
                $jsonResponse = $download_response->json();
          
        if ($download_response->successful() && $jsonResponse['success']) {
        
            $awbLink = $jsonResponse['detail']['awb_link'];
            // $download_order->label=$awbLink;
            // $download_order->save();
            return response()->json(['awb_link' => $awbLink]); 
            
        } else {
        
            if ($download_response->failed()) {
                Log::error('Failed to fetch AWB PDF: ' . $download_response->status());
            }
            return back()->with('error', 'Failed to fetch AWB PDF. Please try again later.');
        }

        }

}


// public function getDownloadLabelAllReturn(Request $request)
// {   
//     $lineItemId=$request->input('lineItemId');
//     $orderId=$request->input('orderId');
//     //dd($orderIds);
//     $session = $request->get('shopifySession');
//     $domain = $session->getShop();
//     $accessToken = $session->getAccessToken();
//     $shop = Session::where('shop', $domain)
//         ->where('access_token', $accessToken)
//         ->first();

//     if (!$shop) {
//         return response()->json(['error' => 'Session not found'], 404);
//     }

//     $accessDetail = $shop->accessDetail;
//     $token = $accessDetail->api_key;
//     $pdf_size=(string)$accessDetail->pdf_size;

//     $pdfFiles = [];

  
//         $download_order = ReturnOrderDetail::where('access_id', $accessDetail->id)
//                                             ->where('id', $orderId)
//                                             ->first();

//          if ($download_order) {

//             $awbNoDetails = json_decode($download_order->awb_number_details, true);
//            // dd($awbNoDetails);
//             $awbNo = null;
//             foreach ($awbNoDetails as $detail) {
//                 if ($detail['line_item_id'] == $lineItemId) {
//                     $awbNo = $detail['awb_number'];
                   

         
//             $download_response = Http::withHeaders([
//                 'token' => $token,
//                 'Accept' => 'application/json',
//             ])->post('https://ferointegrationuat.feroai.com/api/v1/integration/awb_pdf/', [
//                 "awb_number" =>  $awbNo,
//                 "pdf_size" => $pdf_size
//             ]);

//             Log::info($download_response);
//             $jsonResponse = $download_response->json();
//             //dd($jsonResponse);

//             if ($download_response->successful() && $jsonResponse['success']) {
//                 $awbLink = $jsonResponse['detail']['awb_link'];
//                 $pdfContent = file_get_contents($awbLink);
//                 //$pdfContent = $download_response->body();
//                 $fileName = 'pdfs/' . uniqid() . '.pdf';
//                 Storage::put($fileName, $pdfContent);
//                 $pdfFiles[] = storage_path('app/' . $fileName);
//                //dd($pdfFiles);
//                 //$download_order->label = $awbLink;
//                 //$download_order->save();
//             } else {
//                 if ($download_response->failed()) {
//                     Log::error('Failed to fetch AWB PDF: ' . $download_response->status());
//                 }
//                 return response()->json(['error' => 'Failed to fetch AWB PDF. Please try again later.'], 500);
//             }
//         }
//     }
//    }

//     if (empty($pdfFiles)) {
//         return response()->json(['error' => 'No PDFs to merge.'], 400);
//     }

    
//     $mergedPdf = $this->mergePdfs($pdfFiles);
//     //dd($mergedPdf);
//     // Clean up the temporary PDF files
//     foreach ($pdfFiles as $file) {
//         @unlink($file);
//     }

//     $base64Pdf = base64_encode($mergedPdf);

//     return response($mergedPdf, 200)
//         ->header('Content-Type', 'application/pdf')
//         ->header('Content-Disposition', 'attachment; filename="merged.pdf"');

//     // return response()->json([
//     //     'success' => true,
//     //     'pdf' => $base64Pdf,
//     //     'filename' => 'merged.pdf'
//     // ]);
  

// }


public function getDownloadLabelAllReturn(Request $request)
{   
    $lineItemIds = $request->input('lineItemId');
    $orderId = $request->input('orderId');
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $accessDetail = $shop->accessDetail;
    $token = $accessDetail->api_key;

    $pdfFiles = [];

    $download_order = ReturnOrderDetail::where('access_id', $accessDetail->id)
                                        ->where('id', $orderId)
                                        ->first();

    if ($download_order) {
        $awbNoDetails = json_decode($download_order->awb_number_details, true);
      
        if (!is_array($lineItemIds)) {
            $lineItemIds = [$lineItemIds];
        }

        foreach ($lineItemIds as $lineItemId) {
            $awbNo = null;
            foreach ($awbNoDetails as $detail) {
                if ($detail['line_item_id'] == $lineItemId) {
                    $awbNo = $detail['awb_number'];
                    break;
                }
            }

            if ($awbNo) {
                $download_response = Http::withHeaders([
                    'token' => $token,
                    'Accept' => 'application/json',
                ])->post('https://ferointegration.feroai.com/api/v1/integration/awb_pdf/', [
                    "awb_number" => $awbNo,
                    "pdf_size" => "a4"
                ]);

                Log::info($download_response);
                $jsonResponse = $download_response->json();

                if ($download_response->successful() && $jsonResponse['success']) {
                    $awbLink = $jsonResponse['detail']['awb_link'];
                    $pdfContent = file_get_contents($awbLink);
                    $fileName = 'pdfs/' . uniqid() . '.pdf';
                    Storage::put($fileName, $pdfContent);
                    $pdfFiles[] = storage_path('app/' . $fileName);
                    
                } else {
                    if ($download_response->failed()) {
                        Log::error('Failed to fetch AWB PDF: ' . $download_response->status());
                    }
                    return response()->json(['error' => 'Failed to fetch AWB PDF. Please try again later.'], 500);
                }
            }
        }
    }
    //dd($pdfFiles);
    if (empty($pdfFiles)) {
        return response()->json(['error' => 'No PDFs to merge.'], 400);
    }

    $mergedPdf = $this->mergePdfs($pdfFiles);

    // Clean up the temporary PDF files
    foreach ($pdfFiles as $file) {
        @unlink($file);
    }

    return response($mergedPdf, 200)
        ->header('Content-Type', 'application/pdf')
        ->header('Content-Disposition', 'attachment; filename="merged.pdf"');
}

private function mergePdfs(array $pdfFiles)
{
    $pdf = new Fpdi();

    foreach ($pdfFiles as $file) {
    // Check if the file is a valid PDF
    if (file_exists($file) && mime_content_type($file) === 'application/pdf') {
    try {
        $pageCount = $pdf->setSourceFile($file);
        for ($i = 1; $i <= $pageCount; $i++) {
            $tplIdx = $pdf->importPage($i);
            $specs = $pdf->getTemplateSize($tplIdx);

            $pdf->AddPage($specs['orientation'], [$specs['width'], $specs['height']]);
            $pdf->useTemplate($tplIdx);
        }
    } catch (\Exception $e) {
        Log::error('Error processing PDF file: ' . $file . ' - ' . $e->getMessage());
        continue; // Skip to the next file
    }
} else {
    Log::error('Invalid PDF file: ' . $file);
}
}

    return $pdf->Output('S');
}


}
