<?php

namespace App\Http\Controllers;

use App\Models\Order;
use GuzzleHttp\Client;
use App\Models\Session;
use App\Models\SkyexDetail;
use App\Models\AccessDetail;
use Illuminate\Http\Request;
use App\Models\ShipperDetail;
use App\Models\ReturnOrderDetail;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\URL;
use libphonenumber\PhoneNumberType;
use libphonenumber\PhoneNumberUtil;
use Illuminate\Support\Facades\Http;
use libphonenumber\NumberParseException;
use GuzzleHttp\Exception\ClientException;
use LynX39\LaraPdfMerger\Facades\PdfMerger;


class SkyExOrderController extends Controller
{

    private function createClient($shop, $accessToken)
    {
    
        $baseUri = "https://{$shop}/admin/api/2023-04/";
    
        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
    

    public function getSkyexOrders(Request $request)
    {
         
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
            ->where('access_token', $accessToken)
            ->first();

            if (!$shop) {
                return response()->json(['error' => 'Session not found'], 404);
            }
            $client = $this->createClient($domain, $accessToken);
            //dd($shop);
            try {
                $shop_id = $shop->id;
                $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
                $accessFeroDetail = AccessDetail::where('shop_id', $shop_id)->first();
                //$accessDetail = $user->id;
                //dd($accessDetail->id);


                  // Update existing orders if both details are available
                  if ($accessFeroDetail && $accessDetail) {
                    Order::where('skyex_access_id', $accessDetail->id)
                        ->whereNull('access_id')
                        ->update(['access_id' => $accessFeroDetail->id]);
                }


                $orders = [];
                $response = $client->get("orders.json");
                $result = json_decode($response->getBody(), true);
                //dd($result);
                $existingOrderIds = Order::pluck('order_id')->toArray();
                //dd($existingOrderIds);
                foreach ($result['orders'] as $order) {
                    if (in_array($order['id'], $existingOrderIds)) {
                        continue;
                    }
            
                    // $isFulfilled = false;
                    // if ($order['fulfillment_status'] == 'fulfilled') {
                    //     $isFulfilled = true;
                    //     break;
                    // }
                
                    $variantData = [];
            
                    foreach ($order['line_items'] as $item) {
                    
                        $productId = $item['product_id'];
                        if($productId)
                        {
                        $response = $client->get("products/$productId.json");
                        $productResult = json_decode($response->getBody(), true);
            
                        foreach ($productResult['product']['variants'] as $variant) {
                            if($item['variant_id']==$variant['id'])
                            {
                            $product_id = $productResult['product']['id'];
                            $inventoryItemId = $variant['inventory_item_id'];
                            $inventoryResponse = $client->get("inventory_items/$inventoryItemId.json");
                            $inventoryItem = json_decode($inventoryResponse->getBody(), true);
            
                            $variantData[] = [
                                'product_id' => $product_id,
                                'harmonized_system_code' => $inventoryItem['inventory_item']['harmonized_system_code'] ?? null,
                            ];
                        }
                        }
                    }
                    }
        
                   


                         //if (!$isFulfilled) {
                           if ($order['fulfillment_status'] == null) {
                         $order_data = [
                            'order_id' => $order['id'],
                            'order_no'=>$order['order_number'],
                            // 'access_id' => null,
                            'access_id' => $accessFeroDetail ? $accessFeroDetail->id : null,
                            // 'skyex_access_id' => $accessDetail->id,
                            'skyex_access_id' => $accessDetail ? $accessDetail->id : null,
                            'order_data' => $order,
                            'line_items' => $order['line_items'],
                            'hs_codes' => $variantData, 
                            'status' => 0, 
                            ];
            
                            //dd($order_data);
                            Order::create($order_data);
                        }
                }
            
                //dd($orders);
                $orders = Order::
                // where('status', 0)
                where('skyex_access_id', $accessDetail->id)
                    ->where('status', 0)
                    ->where('remove_status', 0)
                    ->orderBy('order_no', 'desc') 
                    ->orderBy('created_at', 'desc')
                    ->get();
        
                   // dd($orders);
                $pushed_orders = Order::where('skyex_access_id', $accessDetail->id)
                ->where('status', 2)
                ->whereNotNull('awb_no')
                ->orderBy('order_no', 'desc') 
                ->orderBy('created_at', 'desc')
                ->get();

                $removed_orders=Order::where('skyex_access_id', $accessDetail->id)
                ->where('status', 0)
                ->where('remove_status', 1)
                ->orderBy('order_no', 'desc') 
                ->orderBy('created_at', 'desc')
                ->get();
                    
                $returned_orders=ReturnOrderDetail::where('access_id', $accessDetail->id)
                ->orderBy('created_at', 'desc')
                ->get();

            return response()->json([
                'orders' => $orders,
                'pushed_orders'=>$pushed_orders,
                'removed_orders'=>$removed_orders,
                'returned_orders'=>$returned_orders
                ], 200);
                    }catch (ClientException $e) {
        
                        Log::error("Error fulfilling order: " . $e->getMessage());
                        return [
                            'success' => false,
                            'message' => "Error fulfilling order: " . $e->getMessage(),
                        ];
                    }
    }
    
    public function skyExGetAWBNumber(Request $request)
    {
        $orderId = $request->input('orderId');
        $selectedCompanyId=$request->input('selectedCompanyId');
      
        $session = $request->get('shopifySession');
        //dd($session);
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
                       ->where('access_token', $accessToken)
                       ->first();
    
        if (!$shop) {
            return response()->json(['error' => 'Session not found'], 404);
        }
        
        $shop_id = $shop->id;
        $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
       // $accessDetail = $shop->accessDetail;
       
        //$shipper_details=ShipperDetail::where('shop_id',$shop->id)->where('id',$selectedCompanyId)->first();

        $order = Order::where('skyex_access_id', $accessDetail->id)
                      ->where('id', $orderId)
                      ->first();
    
        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }
    
        try {
            $awb_number = $this->getShipmentDetails($accessDetail->id, $order,$session);
    
            $pushed_orders = Order::where('skyex_access_id', $accessDetail->id)
                                  ->where('status', 2)
                                  ->whereNotNull('awb_no')
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
            
            $orders = Order::where('skyex_access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('order_no', 'desc') 
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $removed_orders = Order::where('skyex_access_id', $accessDetail->id)
                                  ->where('status', 0)
                                  ->where('remove_status', 1)
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            return response()->json(['orders' => $orders, 'pushed_orders' => $pushed_orders, 'removed_orders' => $removed_orders], 200);
        } catch (\Exception $e) {
            
            Log::error('Error in getAWBNumber: ' . $e->getMessage());
    
            // Retrieve current lists of orders
            $orders = Order::where('skyex_access_id', $accessDetail->id)
                           ->where('status', 0)
                           ->where('remove_status', 0)
                           ->orderBy('order_no', 'desc') 
                           ->orderBy('created_at', 'desc')
                           ->get();
    
            $pushed_orders = Order::where('skyex_access_id', $accessDetail->id)
                                  ->where('status', 2)
                                  ->whereNotNull('awb_no')
                                  ->orderBy('order_no', 'desc') 
                                  ->orderBy('created_at', 'desc')
                                  ->get();
    
            $removed_orders = Order::where('skyex_access_id', $accessDetail->id)
                                   ->where('status', 0)
                                   ->where('remove_status', 1)
                                   ->orderBy('order_no', 'desc') 
                                   ->orderBy('created_at', 'desc')
                                   ->get();
    
            return response()->json([
                'error' => $e->getMessage(),
                'orders' => $orders,
                'pushed_orders' => $pushed_orders,
                'removed_orders' => $removed_orders
            ], 500);
        }
    }


    private function getShipmentDetails($id,$order,$session)
    {

       
            $accessDetail = SkyexDetail::find($id);
           // $session = $accessDetail->sessions;
            //dd('test');
            //dd($session);
            if (!$accessDetail) {
                throw new \Exception('AccessDetail not found');
            }

            $token = $accessDetail->api_key;
            $defaultWeight=$accessDetail->weight;

            $order_data=$order->order_data;
            if (!$order || !$order_data) {
                throw new \Exception('Order data not found or empty');
            }

           
            $ClientReference = !empty($order_data->name) 
                                ? substr($order_data->name, 0, 40) 
                                : null;
            $orderData_shipping_address = $order_data->shipping_address;
            $ConsigneeName =isset($orderData_shipping_address->name) 
                                ? substr($orderData_shipping_address->name, 0, 40) 
                                : null; 
            if (empty($ConsigneeName)) {
                throw new \Exception('Consignee Name is required');
            } 
            $ConsigneeCompanyName = isset($orderData_shipping_address->company) 
                                    ? substr($orderData_shipping_address->company, 0, 40) 
                                    : null;
            $ConsigneeAddress1 = isset($orderData_shipping_address->address1) 
                                    ? substr($orderData_shipping_address->address1, 0, 50) 
                                    : null;               
                                                   
            if (empty($ConsigneeAddress1)) {
                throw new \Exception('Consignee Address is required');
            }                             
            $ConsigneeAddress2 = isset($orderData_shipping_address->address2) 
                                ? substr($orderData_shipping_address->address2, 0, 50) 
                                : null; 
            $ConsigneeAddress3 = '';  
            $ConsigneeCity = isset($orderData_shipping_address->city) 
                                    ? substr($orderData_shipping_address->city, 0, 30) 
                                    : null;                                
            if (empty($ConsigneeCity)) {
                throw new \Exception('Consignee City is required');
            }      
             
            $ConsigneeState = isset($orderData_shipping_address->province) 
                                    ? substr($orderData_shipping_address->province, 0, 40) 
                                    : null;    
                                    
            $ConsigneePostalCode = isset($orderData_shipping_address->zip) 
                                    ? substr($orderData_shipping_address->zip, 0, 16) 
                                    : null;         
            $ConsigneeCountryCode = isset($orderData_shipping_address->country_code) 
                                    ? ($orderData_shipping_address->country_code) 
                                    : null;       
                                   // dd($ConsigneeCountryCode);                         
            if (empty($ConsigneeCountryCode)) {
            throw new \Exception('Consignee Country Code is required');
            }  
            
            $ConsigneeTelephone1 = isset($orderData_shipping_address->phone) 
                                    ? substr($orderData_shipping_address->phone, 0, 50) 
                                    : null;   
            
            $TotalWeightGram = $order_data->total_weight;
            $TotalWeight = $TotalWeightGram / 1000;
            if ($TotalWeight == 0) {
                $TotalWeight = $defaultWeight;
            }
            if ($TotalWeight < 0.01 || $TotalWeight > 1.79769313486232E+308) {
                throw new \Exception('Total weight must be between 0.01 and 1.79769313486232E+308');
            }
            
            $ShipmentType = 1;
            if ($accessDetail->shipper_type == 'NonDocument') {
                $ShipmentType = 2;
              }
            $Length = 0;
            $Width = 0;
            $Height = 0;
            if ($ShipmentType == 2) {
                $dim_w = $TotalWeight * 5000;
                $dim = pow($dim_w, 1 / 3);
                $Length = $dim;
                $Width = $dim;
                $Height = $dim;
            }
            
            $line_items = '';
            $line_items = $order->line_items;
            foreach ($line_items as $line_item) {
                $contents[] = $line_item->name;
            }
            $ContentDescription = implode(", ", $contents);
            $ContentDescription = substr($ContentDescription, 0, 85);
            $ConsignmentValue = $order_data->total_price_set->presentment_money->amount ?? null;
            $ConsignmentValueCurrencyCode = $order_data->total_price_set->presentment_money->currency_code ?? null;
             $note = $order_data->note ?? null;
            $SpecialInstruction = $ContentDescription . '-' . $note;
            $SpecialInstruction = substr($SpecialInstruction, 0, 240);

            

            $payment_gateway=$order_data->payment_gateway_names;
            $payment_terms=$order_data->payment_terms;
            $CODValue = 0.0;
            $CODValueCurrencyCode='';
            // if (empty($payment_gateway) && $payment_terms) {
            //     $CODValue = (float) $order_data->current_total_price;
            //     $CODValueCurrencyCode = $order_data->current_total_price_set->presentment_money->currency_code ?? null;
            // }
            
            // if (is_array($payment_gateway) && in_array("Cash on Delivery (COD)", $payment_gateway)) {
            //     $CODValue = (float) $order_data->current_total_price;
            //     $CODValueCurrencyCode = $order_data->current_total_price_set->presentment_money->currency_code ?? null;
            // }
            
           
            //dd($CODValue,$CODValueCurrencyCode);
        
            try {
                
              
                // $response = Http::withHeaders([
                //     'Authorization' => 'Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpYXQiOiIxNjY2MjcwODk3IiwiaXNzIjoiaHR0cHM6Ly9za3lleHByZXNzaW50ZXJuYXRpb25hbC5jb20iLCJhdWQiOiJodHRwczovL3NreWV4cHJlc3NpbnRlcm5hdGlvbmFsLmNvbSIsImV4cCI6IjE3NTI2NzA4OTciLCJ1bmFtZSI6IlNIT1BJRllURVNUMSIsInVpZCI6IjZjNjdhMTU2MGZjYzQ2YzBiMGU2YTJkZTgwNmNhZmIyIn0.bbfl4_yINfNh4STr49s6eduyIiiWskhSjYhbfIZtlT0',
                //    'Accept' => 'application/json',
                // ])->post('https://www.skyexpressinternational.com/api/Booking', [
                    
                //         "ClientReference" => "#1091",
                //         "ConsigneeName" =>  "Mohammed",
                //         "ConsigneeCompanyName" =>  "MAQ",
                //         "ConsigneeAddress1" =>  "Marina",
                //         "ConsigneeAddress2" =>  "Dubai",
                //         "ConsigneeAddress3" =>  "",
                //         "ConsigneeCity" =>  "Dubai",
                //         "ConsigneeState" =>  "",
                //         "ConsigneePostalCode" =>  "",
                //         "ConsigneeCountryCode" =>  "AE",
                //         "ConsigneeTelephone1" =>  "1234567890",
                //         "ConsigneeFax1" =>  "",
                //         "ReadyAtTime" =>  "",
                //         "ClosingTime" =>  "",
                //         "TotalWeight" =>  13.0,
                //         "Length" =>  14.0,
                //         "Width" =>  15.0,
                //         "Height" =>  16.0,
                //         "NumberOfItemsThisSize" =>  '',
                //         "ItemsCount" =>  1,
                //         "ShipmentType" =>  2,
                //         "ContentDescription" =>  "sample string 19",
                //         "ConsignmentValue" =>  1.0,
                //         "ConsignmentValueCurrencyCode" =>  "AED",
                //         "InsuranceValue" =>  '',
                //         "InsuranceValueCurrencyCode" =>  '',
                //         "SpecialInstruction" =>  "sample string 22",
                //         "CODValue" =>  1.0,
                //         "CODValueCurrencyCode" =>  "AED",
                //         "CustomInvoice_Url" =>  ""
                      
                // ]);


                $body = [
                    "ClientReference" => $ClientReference,
                    "ConsigneeName" =>  $ConsigneeName,
                    "ConsigneeCompanyName" =>  $ConsigneeCompanyName,
                    "ConsigneeAddress1" =>  $ConsigneeAddress1,
                    "ConsigneeAddress2" =>  $ConsigneeAddress2,
                    "ConsigneeAddress3" =>  "",
                    "ConsigneeCity" =>  $ConsigneeCity,
                    "ConsigneeState" =>  $ConsigneeState,
                    "ConsigneePostalCode" =>  $ConsigneePostalCode,
                    "ConsigneeCountryCode" =>  $ConsigneeCountryCode,
                    "ConsigneeTelephone1" =>  $ConsigneeTelephone1,
                    "ConsigneeFax1" =>  "",
                    "ReadyAtTime" =>  "",
                    "ClosingTime" =>  "",
                    "TotalWeight" =>  $TotalWeight,
                    "Length" =>  $Length,
                    "Width" =>  $Width,
                    "Height" =>  $Height,
                    //"NumberOfItemsThisSize": 1,
                    "ItemsCount" =>  1,
                    "ShipmentType" =>  $ShipmentType,
                    "ContentDescription" =>  $ContentDescription,
                    "ConsignmentValue" =>  $ConsignmentValue,
                    "ConsignmentValueCurrencyCode" =>  $ConsignmentValueCurrencyCode,
                    //"InsuranceValue"=> 1.0,
                    //"InsuranceValueCurrencyCode"=> "AED",
                     "SpecialInstruction" =>  $SpecialInstruction
                ];
                
                // Check if COD value and currency need to be added
                if (empty($payment_gateway) && $payment_terms) {
                    $CODValue = (float) $order_data->current_total_price;
                    $CODValueCurrencyCode = $order_data->current_total_price_set->presentment_money->currency_code ?? null;
                
                    $body['CODValue'] = $CODValue;
                    $body['CODValueCurrencyCode'] = $CODValueCurrencyCode;
                } elseif (is_array($payment_gateway) && in_array("Cash on Delivery (COD)", $payment_gateway)) {
                    $CODValue = (float) $order_data->current_total_price;
                    $CODValueCurrencyCode = $order_data->current_total_price_set->presentment_money->currency_code ?? null;
                
                    $body['CODValue'] = $CODValue;
                    $body['CODValueCurrencyCode'] = $CODValueCurrencyCode;
                }
                
                $response = Http::withToken($token)->post('https://www.skyexpressinternational.com/api/Booking', $body);
                

                $response_body = json_decode($response->body(), true);
                //dd($response_body);
                Log::info($response_body);

                if (!empty($response_body['AWBNo'])) {
                    $awb_number = $response_body['AWBNo'] ?? null;
                     //dd($awb_number);
                   
                    Log::info($awb_number);

                    // $er = '';
                    // $er1 = 0;
                    if (!empty($awb_number)) {

                        $label = Http::withToken($token)->get('https://www.skyexpressinternational.com/api/AWB/' . $awb_number);
            
                        // $pdf_directory = public_path('pdf');
                        // $pdf_name = 'skyex' . time() . '.pdf';
                        // $full_path = $pdf_directory . '/' . $pdf_name;

                        // // Check if the directory exists, if not, create it
                        // if (!is_dir($pdf_directory)) {
                        //     mkdir($pdf_directory, 0755, true); // Create the directory with proper permissions
                        // }

                        // // Write the PDF file
                        // file_put_contents($full_path, $label->body());

            
                        // $pdf_url = url('public/pdf/' . $pdf_name);
                        // //dd($pdf_url,$awb_number,$order,$session,$awb_number);
                        
            
                        // $pdf_name = 'skyex' . time() . '.pdf';
                        $pdf_name = 'skyex' . time() .'_'. $awb_number . '.pdf';
                        $full_path = public_path() . '/pdf/' . $pdf_name;
                        file_put_contents($full_path, $label->body());
            
                        $pdf_url = url('public/pdf/' . $pdf_name);

                        $order->awb_no = $awb_number;
                        $order->label = $pdf_url;
                        $order->status = 2;
                        $order->save();
                        //dd($order,$session,$awb_number);
                        $fulfillment=$this->getFulfillment($order,$session,$awb_number);

                        $body = array($awb_number);
                        $ready = Http::withToken($token)->post('https://www.skyexpressinternational.com/api/Booking/ReadyForPickup', $body);
                        $ready_body = $ready->body();
                        $ready_status = $ready->status();

            
                        if ($ready_status == 200) {
                            $order->pickup_status = 'Ready For PickUp';
                            $order->save();
                        }
                        

                       // $er1 = 0;

                      
            
                    } 
                    // else {
                    //     $er1 = $er1 + 1;
                    //     // $er = 'The request is invalid. Please fix the errors and then try again.';
                    //    // $er .= $name . ':';
                    //     if (!empty($response_body['ModelError'])) {
                    //       foreach ($response_body['ModelError'] as $ModelError) {
                    //         $er .= end($ModelError);
                    //       }
                    //       // return response()->json($er);
                    //     }
                    //   }
                     

                } else {
                    $er = '';
                    if (!empty($response_body['ModelError'])) {
                        foreach ($response_body['ModelError'] as $ModelError) {
                          $er .= end($ModelError);
                        }
                        // return response()->json($er);
                      }

                    //$errorMessage = $response_body['Message'] ?? 'Unknown error occurred.';
                    $errorMessage = $er ?? 'Unknown error occurred.';
                    throw new \Exception($errorMessage);
                    //return response()->json(['error' => $errorMessage], $response->status());
        }
            } catch (\Exception $e) {
                throw new \Exception($e->getMessage());
                //return response()->json(['error' => $e->getMessage()], 500);
            }
    } 
    

    private function getFulfillment($order, $session, $awb_number)
    {
        // $shop = $session->shop;
        $shop = $session->getShop(); 
       
        $orderId = $order->order_id;
        // $accessToken = $session->access_token;
        $accessToken = $session->getAccessToken(); 
        //dd($accessToken);
        $client = $this->createClient($shop, $accessToken);
    
        try {
            
            $response = $client->get("orders/{$orderId}/fulfillment_orders.json");
            $result = json_decode($response->getBody(), true);
            Log::info(json_encode($result));
    
           
            if (isset($result['fulfillment_orders']) && !empty($result['fulfillment_orders'])) {
                $fulfillment_order = $result['fulfillment_orders'][0];
                $fulfillment_id = $fulfillment_order['id'];
    
               
                $response = $client->post('fulfillments.json', [
                    'json' => [
                        'fulfillment' => [
                            'line_items_by_fulfillment_order' => [
                                [
                                    'fulfillment_order_id' => $fulfillment_id,
                                ]
                            ],
                            'tracking_info' => [
                                'number' => $awb_number,
                            ],
                            'message' => "Your custom message here",
                            'notify_customer' => true, 
                        ]
                    ]
                ]);
    
                
                $statusCode = $response->getStatusCode();
                
                if ($statusCode >= 200 && $statusCode <= 299) {
                    return [
                        'success' => true,
                        'message' => "Fulfillment created successfully.",
                    ];
                } else {
                    return [
                        'success' => false,
                        'message' => "Failed to create fulfillment. Status code: {$statusCode}",
                    ];
                }
            } else {
                Log::warning("No fulfillment orders found for order ID: {$orderId}");
                return [
                    'success' => false,
                    'message' => "No fulfillment orders found for the order.",
                ];
            }
        } catch (ClientException $e) {
           
            Log::error("Error fulfilling order: " . $e->getMessage());
            return [
                'success' => false,
                'message' => "Error fulfilling order: " . $e->getMessage(),
            ];
        } catch (\Exception $e) {
            
            Log::error("Exception: " . $e->getMessage());
            return [
                'success' => false,
                'message' => "Exception: " . $e->getMessage(),
            ];
        }
    }



    // public function getDownloadLabelSkyEx(Request $request)
    // {
    //         $id=$request->input('orderId');
    //         $session = $request->get('shopifySession');
    //         $domain = $session->getShop();
    //         $accessToken = $session->getAccessToken();
    //         $shop = Session::where('shop', $domain)
    //             ->where('access_token', $accessToken)
    //             ->first();
    
    //             if (!$shop) {
    //                 return response()->json(['error' => 'Session not found'], 404);
    //             }
    
            
    //         $shop_id = $shop->id;
    //         $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
            
    //         $token = $accessDetail->api_key;
    //         //dd($token);
    //         $download_order = Order::where('skyex_access_id', $accessDetail->id)
    //         ->where('status', 1)
    //         ->where('id', $id)
    //         ->first();
    
    //         if($download_order)
    //         {
    //             $awbNo=$download_order->awb_no;
    //             if ($awbNo) {

    //                 // $download_response = Http::withToken($token)->get('https://www.skyexpressinternational.com/api/AWB/' . $awbNo);
        
    //                 // dd($download_response);
    //                 $label =  $download_order->label;
    //                 $pdf_directory = public_path('pdf');
    //                 $pdf_name = 'skyex' . time() . '.pdf';
    //                 $full_path = $pdf_directory . '/' . $pdf_name;

    //                 // Check if the directory exists, if not, create it
    //                 if (!is_dir($pdf_directory)) {
    //                     mkdir($pdf_directory, 0755, true); // Create the directory with proper permissions
    //                 }

    //                 // Write the PDF file
    //                 // file_put_contents($full_path, $download_response->body());
    //                 file_put_contents($full_path, $label);
        
    //                 $awbLink = url('public/pdf/' . $pdf_name);

    //                 //dd($awbLink);
    //                 // $download_order->label=$awbLink;
    //                 // $download_order->save();

    //                 // $pdfMerger = PDFMerger::init(); //Initialize the merger


    //                 // $file_ar = explode('/', $awbLink);
    //                 // $file = end($file_ar);
    //                 // if (!empty($file)) {
    //                 //   $file_path = base_path('public/pdf/' . $file);
    //                 //   if (file_exists($file_path)) {
    //                 //     $pdfMerger->addPDF($file_path, 'all');
    //                 //   } 
    //                 // } 

    //                 // $pdfMerger->merge();
    //                 // $fpath = url('public/pdf/skyexSelected' . time() .'.pdf');
    //                 // $pdfMerger->save(public_path('pdf/skyexSelected' . '.pdf'), "file");


    //                 return response()->json(['awb_link' => $awbLink]); 
    //                 //return response()->download($full_path, $pdf_name)->deleteFileAfterSend(true);
                
    //             } else {
    //                 return back()->with('error', 'Failed to fetch AWB PDF. Please try again later.');
    //             }
    
    //         }

           
    
    // }


    public function getDownloadLabelSkyEx(Request $request)
{
    $id = $request->input('orderId');
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $shop_id = $shop->id;
    $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
    
    $token = $accessDetail->api_key;
    $download_order = Order::where('skyex_access_id', $accessDetail->id)
        ->where('status', 2)
        ->where('id', $id)
        ->first();

   if ($download_order && $download_order->awb_no) {
    $label = $download_order->label;
    $file_ar = explode('/', $label);
    $file = end($file_ar);
    
    $file_path = public_path('pdf/' . $file);
    
    if (file_exists($file_path)) {
        return response()->file($file_path, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'attachment; filename=' . $file,
        ])->deleteFileAfterSend(false);
    } else {
        return response()->json(['error' => 'PDF file not found'], 404);
    }

    } else {
        return response()->json(['error' => 'Failed to fetch AWB PDF. Please try again later.'], 400);
    }
}

//     public function getDownloadLabelAllSkyEx(Request $request)
// {
//     $orderIds = $request->input('orderIds'); 
//     $session = $request->get('shopifySession');
//     $domain = $session->getShop();
//     $accessToken = $session->getAccessToken();
//     $shop = Session::where('shop', $domain)
//         ->where('access_token', $accessToken)
//         ->first();

//     if (!$shop) {
//         return response()->json(['error' => 'Session not found'], 404);
//     }

//     $shop_id = $shop->id;
//     $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
//     $token = $accessDetail->api_key;

//     $pdfMerger = PDFMerger::init(); // Initialize the merger at the beginning

//     foreach ($orderIds as $id) {
//         $download_order = Order::where('skyex_access_id', $accessDetail->id)
//             ->where('status', 1)
//             ->where('id', $id)
//             ->first();

//         if($download_order) {
//             $awbNo = $download_order->awb_no;
//             if ($awbNo) {
//                 // Fetch the PDF from SkyEx API
//                 // $download_response = Http::withToken($token)->get('https://www.skyexpressinternational.com/api/AWB/' . $awbNo);

//                 // $pdf_directory = public_path('pdf');
//                 // $pdf_name = 'skyex_' . time() . '.pdf'; // Use AWB No and time to generate unique names
//                 // $full_path = $pdf_directory . '/' . $pdf_name;

//                 // // Check if the directory exists, if not, create it
//                 // if (!is_dir($pdf_directory)) {
//                 //     mkdir($pdf_directory, 0755, true); // Create the directory with proper permissions
//                 // }

//                 // // Write the PDF file
//                 // file_put_contents($full_path, $download_response->body());

//                 // // Save the PDF link in the order
//                 // $awbLink = url('public/pdf/' . $pdf_name);
//                 // $download_order->label = $awbLink;
//                 // $download_order->save();

//                 // // Add the current PDF to the merger
//                 // $pdfMerger->addPDF($full_path, 'all'); // Add the newly created PDF for the current order


//                 $file_ar = explode('/', $download_order->label);
//                 $file = end($file_ar);
//                 //dd($file_ar,$file);
                
//                   $file_path = base_path('public/pdf/' . $file);
//                   if (file_exists($file_path)) {
//                     $pdfMerger->addPDF($file_path, 'all');
//                   } 


//             } else {
//                 return back()->with('error', 'Failed to fetch AWB PDF. Please try again later.');
//             }
//         }
//     }

//     // After all orders are processed, merge the PDFs
//     //$merged_pdf_name = 'skyexSelected_' . $shop->id . '_' . time() . '.pdf';
//     $merged_pdf_name = 'skyexMergedPdf' . $shop->id .'.pdf';
//     $merged_pdf_path = public_path('pdf/' . $merged_pdf_name);

//     // Merge all added PDFs and save to the final file
//     $pdfMerger->merge();
//     $pdfMerger->save($merged_pdf_path, "file");

//     // Return the link to the merged PDF
//     $fpath = url('public/pdf/' . $merged_pdf_name);
//     //dd($fpath);
//     return response()->json(['awb_link' => $fpath]);
// }


public function getDownloadLabelAllSkyEx(Request $request)
{
    $orderIds = $request->input('orderIds'); 
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $shop_id = $shop->id;
    $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
    $token = $accessDetail->api_key;

    $pdfMerger = PDFMerger::init();

    foreach ($orderIds as $id) {
        $download_order = Order::where('skyex_access_id', $accessDetail->id)
            ->where('status', 2)
            ->where('id', $id)
            ->first();

        if($download_order && $download_order->awb_no) {
            $file_ar = explode('/', $download_order->label);
            $file = end($file_ar);
            
            $file_path = public_path('pdf/' . $file);
            if (file_exists($file_path)) {
                $pdfMerger->addPDF($file_path, 'all');
            } 
        }
    }

    $merged_pdf_name = 'skyexMergedPdf' . $shop->id . '_' . time() . '.pdf';
    $merged_pdf_path = public_path('pdf/' . $merged_pdf_name);

    $pdfMerger->merge();
    $pdfMerger->save($merged_pdf_path, "file");

    // Return the file content instead of the URL
    return response()->file($merged_pdf_path)->deleteFileAfterSend(true);
}



    public function getRemoveAllSkyEx(Request $request)
{
    $session = $request->get('shopifySession');
    $domain = $session->getShop();
    $accessToken = $session->getAccessToken();
    $shop = Session::where('shop', $domain)
        ->where('access_token', $accessToken)
        ->first();

    if (!$shop) {
        return response()->json(['error' => 'Session not found'], 404);
    }

    $orderIds = $request->input('orderIds');

    $shop_id = $shop->id;
    $accessDetail = SkyexDetail::where('shop_id', $shop_id)->first();
    
    
    foreach ($orderIds as $id) {
        $remove_order = Order::where('skyex_access_id', $accessDetail->id)
            ->where('id', $id)
            ->first();

        if ($remove_order) {
            $remove_order->remove_status = 1;
            $remove_order->save(); 
        }
    }

    $orders = Order::where('skyex_access_id', $accessDetail->id)
        ->where('status', 0)
        ->where('remove_status', 0)
        ->orderBy('order_no', 'desc') 
        ->orderBy('created_at', 'desc')
        ->get();

    $removed_orders = Order::where('skyex_access_id', $accessDetail->id)
         ->where('status', 0)
        ->where('remove_status', 1)
        ->orderBy('created_at', 'desc')
        ->get();


    return response()->json([
        'orders' => $orders,
        'removed_orders' => $removed_orders
    ], 200);
}


}
