<?php

namespace App\Http\Controllers;

use GuzzleHttp\Client;
use App\Models\Session;
use App\Models\SkyexDetail;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;

class SkyexDetailsController extends Controller
{

    private function createClient($session)
    {
        $shop = $session->getShop();
        $accessToken = $session->getAccessToken();
        $baseUri = "https://{$shop}/admin/api/2023-01/";

        return new Client([
            'base_uri' => $baseUri,
            'headers' => [
                'Content-Type' => 'application/json',
                'X-Shopify-Access-Token' => $accessToken,
            ],
        ]);
    }
    // public function saveskyexdetial(Request $request)
    // {
    //     // Validate the incoming request
    //     $validated = $request->validate([
    //         'skyexApikey' => 'required|string',
    //         'weight' => 'required|numeric',
    //         'shipperType' => 'required|string',
    //     ]);
      
    //     try {
    //         // Get Shopify session details
    //         $session = $request->get('shopifySession');
    //         $domain = $session->getShop();
    //         $accessToken = $session->getAccessToken();
    
    //         $shop = Session::where('shop', $domain)
    //             ->where('access_token', $accessToken)
    //             ->first();
    
    //         if (!$shop) {
    //             return response()->json(['error' => 'Session not found'], 404);
    //         }
    
    //         $shop_id = $shop->id;
    //         $skyexApikey = $validated['skyexApikey'];
    //         $weight = $validated['weight'];
    //         $shipperType = $validated['shipperType'];
    
    //         // Log request details
    //         Log::info('Validating SkyEx API key', ['api_key' => $skyexApikey]);
    
    //         // Validate the API key with SkyEx API
    //         $response = Http::withToken($skyexApikey)->get('https://www.skyexpressinternational.com/api/Booking/123');

    //         //dd($response);

    
    //         if ($response->status() == '401') {
    //             // Invalid API key
    //             return response()->json(['error' => 'Invalid SkyEx API key. Please provide a valid key.'], 401);
    //         }
    //        // dd($response);
    //         // API key is valid, proceed with saving the details
    //         SkyexDetail::updateOrCreate(
    //             ['shop_id' => $shop_id],
    //             [
    //                 'api_key' => $skyexApikey,
    //                 'shipper_type' => $shipperType,
    //                 'weight' => $weight,
    //                 'status' => $shop_id,
    //             ]
    //         );
    
    //         return response()->json(['success' => 'SkyEx details saved successfully']);
    //     } catch (\Exception $e) {
    //         // Log the exception for further debugging
    //         Log::error('Error saving SkyEx details', ['message' => $e->getMessage()]);
    //         return response()->json(['error' => 'Internal Server Error. Please try again later.'], 500);
    //     }
    // }

public function saveskyexdetial(Request $request)
    {
        // Validate the incoming request
        $validated = $request->validate([
            'skyexApikey' => 'required|string',
            'weight' => 'required|numeric',
            'account' => 'required|string',
            'shipperType' => 'required|string',
        ]);
    
        try {
            // Get Shopify session details
            $session = $request->get('shopifySession');
            $domain = $session->getShop();
            $accessToken = $session->getAccessToken();
    
            $shop = Session::where('shop', $domain)
                ->where('access_token', $accessToken)
                ->first();
    
            if (!$shop) {
                return response()->json(['error' => 'Session not found'], 404);
            }
    
            $shop_id = $shop->id;
            $skyexApikey = $validated['skyexApikey'];
            $weight = $validated['weight'];
            $account = $validated['account'];
            $shipperType = $validated['shipperType'];
    
            // Log request details
            Log::info('Validating SkyEx API key', ['api_key' => $skyexApikey]);
    
            // Validate the API key with SkyEx API
            $response = Http::withToken($skyexApikey)->get('https://www.skyexpressinternational.com/api/Booking/123');
    
            // If the API key is invalid
            if ($response->status() == 401) {
                // Fetch the old data to return in the response
                $oldData = SkyexDetail::where('shop_id', $shop_id)->first();
    
                // Invalid API key response with old data
                return response()->json([
                    'error' => 'Invalid SkyEx API key. Please provide a valid key.',
                    'oldData' => $oldData
                ], 401);
            }
    
            // API key is valid, proceed with saving the details
            SkyexDetail::updateOrCreate(
                ['shop_id' => $shop_id],
                [
                    'account' => $account,
                    'api_key' => $skyexApikey,
                    'shipper_type' => $shipperType,
                    'weight' => $weight,
                    'status' => 1,
                ]
            );

            $this->addApiStatus($request);
    
            return response()->json(['success' => 'SkyEx details saved successfully']);
        } catch (\Exception $e) {
            // Log the exception for further debugging
            Log::error('Error saving SkyEx details', ['message' => $e->getMessage()]);
    
            // Fetch the old data to return in the response
            $oldData = SkyexDetail::where('shop_id', $shop_id)->first();
    
            return response()->json([
                'error' => 'Internal Server Error. Please try again later.',
                'oldData' => $oldData
            ], 500);
        }
    }
    
    
    public function addApiStatus(Request $request)
    {
        $session = $request->get('shopifySession');
        $client = $this->createClient($session);
        $response = $client->get('shop.json');
        $domain = $session->getShop();
    
        DB::table('sessions')
            ->where('shop', $domain)
            ->update(['api_status' => 2]);
    
        //$shop = DB::table('sessions')->where('shop', $domain)->first();
        //dd($shop->api_status); // This will display the updated api_status
    }
    

public function getDetails(Request $request)
{

        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
            ->where('access_token', $accessToken)
            ->first();
        $shop_id = $shop->id;
        $data = SkyexDetail::where('shop_id', $shop_id)->first();

        return response()->json($data);
    }

    public function deleteDetails(Request $request){
        $session = $request->get('shopifySession');
        $domain = $session->getShop();
        $accessToken = $session->getAccessToken();
        $shop = Session::where('shop', $domain)
            ->where('access_token', $accessToken)
            ->first();
        $shop_id = $shop->id;
        $data = SkyexDetail::where('shop_id', $shop_id)->delete();
        return response()->json($data);

}

    public function checkSkyexTableStatus(Request $request)
{
    try {
        // Ensure 'shopifySession' exists in the request
        $session = $request->get('shopifySession');
        if (!$session) {
            return response()->json(['error' => 'Shopify session not provided'], 400);
        }

        // Create the Shopify client
        $client = $this->createClient($session);

        // Make the request to Shopify
        $response = $client->get('shop.json');
        $shop_details = json_decode($response->getBody(), true);

        // Get the shop domain and retrieve the shop record from the database
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
        $apiStatusDetails = $shop -> api_status;

        if (!$shop) {
            return response()->json(['error' => 'Shop not found in the database'], 404);
        }

        // Get the shop ID and retrieve user details from AccessDetail
        $shop_id = $shop->id;
       
        $skyexUserDetails = SkyexDetail::where('shop_id', $shop_id)->first();
        //dd($skyexUserDetails);
        if ($skyexUserDetails) {
            $status = $skyexUserDetails->status;
          
          

            $data = [
              //  'status' => $status,
               'skyexStatus' => $status,
            
                ];
            return response()->json($data, 200);
        } else {
            return response()->json('not found', 200);
        }
    } catch (\Exception $e) {
        // Handle any unexpected exceptions
        return response()->json(['error' => 'An error occurred', 'message' => $e->getMessage()], 500);
    }
}


public function checkApiStatusdata(Request $request)
{
    try {
        // Ensure 'shopifySession' exists in the request
        $session = $request->get('shopifySession');
        if (!$session) {
            return response()->json(['error' => 'Shopify session not provided'], 400);
        }

        // Create the Shopify client
        $client = $this->createClient($session);

        // Make the request to Shopify
        $response = $client->get('shop.json');
        $shop_details = json_decode($response->getBody(), true);

        // Get the shop domain and retrieve the shop record from the database
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
        $apiStatusDetails = $shop -> api_status;

        if (!$shop) {
            return response()->json(['error' => 'Shop not found in the database'], 404);
        }

      
        // if ($apiStatusDetails) {
           
           $apiStatus = $apiStatusDetails;
           // $skyexStatus =  $skyexUserDetails;

            $data = [
                'apiStatus' => $apiStatus
                ];
            return response()->json($data, 200);
        // } else {
        //     return response()->json('not found', 200);
        // }
    } catch (\Exception $e) {
        // Handle any unexpected exceptions
        return response()->json(['error' => 'An error occurred', 'message' => $e->getMessage()], 500);
    }
}


public function checkChangeApiStatusdata(Request $request)
{
    try {
        // Ensure 'shopifySession' exists in the request
        $session = $request->get('shopifySession');
        if (!$session) {
            return response()->json(['error' => 'Shopify session not provided'], 400);
        }

        // Create the Shopify client
        $client = $this->createClient($session);

        // Make the request to Shopify
        $response = $client->get('shop.json');
        $shop_details = json_decode($response->getBody(), true);

        // Get the shop domain and retrieve the shop record from the database
        $domain = $session->getShop();
        $shop = DB::table('sessions')->where('shop', $domain)->first();
        $apiStatusDetails = $shop -> buttonChangeApiStatus;

        if (!$shop) {
            return response()->json(['error' => 'Shop not found in the database'], 404);
        }

      
        // if ($apiStatusDetails) {
           
           $apiStatus = $apiStatusDetails;
           // $skyexStatus =  $skyexUserDetails;

            $data = [
                'buttonChangeApiStatus' => $apiStatus
                ];
            return response()->json($data, 200);
        // } else {
        //     return response()->json('not found', 200);
        // }
    } catch (\Exception $e) {
        // Handle any unexpected exceptions
        return response()->json(['error' => 'An error occurred', 'message' => $e->getMessage()], 500);
    }
}

}
